# -*- coding: UTF-8 -*-
'''
    product

    :copyright: (c) 2013-2015 by Openlabs Technologies & Consulting (P) Limited
    :license: BSD, see LICENSE for more details.
'''
from decimal import Decimal

from trytond.model import ModelView, fields
from trytond.transaction import Transaction
from trytond.wizard import Wizard, StateView, StateTransition, Button
from trytond.pool import PoolMeta, Pool


__all__ = [
    'Product', 'ExportAmazonCatalogStart', 'ExportAmazonCatalog',
    'ExportAmazonCatalogDone', 'ExportAmazonPricingStart',
    'ExportAmazonPricing', 'ExportAmazonPricingDone',
    'ExportAmazonInventoryStart', 'ExportAmazonInventory',
    'ExportAmazonInventoryDone', 'ProductCode',
    'Template',
]
__metaclass__ = PoolMeta


class Template:
    "Product Template"
    __name__ = 'product.template'

    export_to_amazon = fields.Boolean('Amazon Exportable')


class Product:
    "Product"
    __name__ = "product.product"

    asin = fields.Function(fields.Many2One(
        'product.product.code', 'ASIN'
    ), 'get_codes')
    ean = fields.Function(fields.Many2One(
        'product.product.code', 'EAN'
    ), 'get_codes')
    upc = fields.Function(fields.Many2One(
        'product.product.code', 'UPC'
    ), 'get_codes')
    isbn = fields.Function(fields.Many2One(
        'product.product.code', 'ISBN'
    ), 'get_codes')
    gtin = fields.Function(fields.Many2One(
        'product.product.code', 'GTIN'
    ), 'get_codes')

    @classmethod
    def get_codes(cls, products, names):
        ProductCode = Pool().get('product.product.code')

        res = {}
        for name in names:
            res[name] = {}
            for product in products:
                code = ProductCode.search([
                    ('product', '=', product.id),
                    ('code_type', '=', name)
                ])
                res[name][product.id] = code and code[0].id or None

        return res

    @classmethod
    def find_or_create_using_amazon_sku(cls, sku):
        """
        Find or create a product using Amazon Seller SKU. This method looks
        for an existing product using the SKU provided. If found, it
        returns the product found, else creates a new one and returns that

        :param sku: Product Seller SKU from Amazon
        :returns: Active record of Product Created
        """
        SaleChannel = Pool().get('sale.channel')

        products = cls.search([('code', '=', sku)])

        if products:
            return products[0]

        # if product is not found get the info from amazon and
        # delegate to create_using_amazon_data
        amazon_channel = SaleChannel(
            Transaction().context['current_channel']
        )
        assert amazon_channel.source == 'amazon_mws'

        api = amazon_channel.get_amazon_product_api()

        product_data = api.get_matching_product_for_id(
            amazon_channel.amazon_marketplace_id, 'SellerSKU', [sku]
        ).parsed

        return cls.create_using_amazon_data(product_data)

    @classmethod
    def extract_product_values_from_amazon_data(cls, product_attributes):
        """
        Extract product values from the amazon data, used for
        creation of product. This method can be overwritten by
        custom modules to store extra info to a product

        :param product_data: Product data from amazon
        :returns: Dictionary of values
        """
        SaleChannel = Pool().get('sale.channel')

        amazon_channel = SaleChannel(
            Transaction().context['current_channel']
        )
        assert amazon_channel.source == 'amazon_mws'

        return {
            'name': product_attributes['Title']['value'],
            'list_price': Decimal('0.01'),
            'cost_price': Decimal('0.01'),
            'default_uom': amazon_channel.default_uom.id,
            'salable': True,
            'sale_uom': amazon_channel.default_uom.id,
            'account_expense': amazon_channel.default_account_expense.id,
            'account_revenue': amazon_channel.default_account_revenue.id,
        }

    @classmethod
    def create_using_amazon_data(cls, product_data):
        """
        Create a new product with the `product_data` from amazon.

        :param product_data: Product Data from Amazon
        :returns: Active record of product created
        """
        Template = Pool().get('product.template')

        # TODO: Handle attribute sets in multiple languages
        product_attribute_set = product_data['Products']['Product'][
            'AttributeSets'
        ]
        if isinstance(product_attribute_set, dict):
            product_attributes = product_attribute_set['ItemAttributes']
        else:
            product_attributes = product_attribute_set[0]['ItemAttributes']

        product_values = cls.extract_product_values_from_amazon_data(
            product_attributes
        )

        product_values.update({
            'products': [('create', [{
                'code': product_data['Id']['value'],
                'description': product_attributes['Title']['value'],
                'channel_listings': [('create', [{
                    # TODO: Set product identifier
                    'channel': Transaction().context['current_channel']
                }])]
            }])],
        })

        product_template, = Template.create([product_values])

        return product_template.products[0]


class ProductCode:
    "Amazon Product Identifier"
    __name__ = 'product.product.code'

    @classmethod
    def __setup__(cls):
        """
        Setup the class before adding to pool
        """
        super(ProductCode, cls).__setup__()
        cls.code_type.selection.extend([
            ('upc', 'UPC'),
            ('isbn', 'ISBN'),
            ('asin', 'ASIN'),
            ('gtin', 'GTIN')
        ])


class ExportAmazonCatalogStart(ModelView):
    'Export Catalog to Amazon View'
    __name__ = 'amazon.export_catalog.start'


class ExportAmazonCatalogDone(ModelView):
    'Export Catalog to Amazon Done View'
    __name__ = 'amazon.export_catalog.done'

    status = fields.Char('Status', readonly=True)
    submission_id = fields.Char('Submission ID', readonly=True)


class ExportAmazonCatalog(Wizard):
    '''Export catalog to Amazon

    Export the products selected to this amazon account
    '''
    __name__ = 'amazon.export_catalog'

    start = StateView(
        'amazon.export_catalog.start',
        'amazon_mws.export_catalog_start', [
            Button('Cancel', 'end', 'tryton-cancel'),
            Button('Continue', 'export_', 'tryton-ok', default=True),
        ]
    )
    export_ = StateTransition()
    done = StateView(
        'amazon.export_catalog.done',
        'amazon_mws.export_catalog_done', [
            Button('OK', 'end', 'tryton-cancel'),
        ]
    )

    def transition_export_(self):
        """
        Export the products selected to this amazon account
        """
        SaleChannel = Pool().get('sale.channel')

        amazon_channel = SaleChannel(Transaction().context.get('active_id'))

        response = amazon_channel.export_catalog_to_amazon()

        Transaction().set_context({'response': response})

        return 'done'

    def default_done(self, fields):
        "Display response"
        response = Transaction().context['response']
        return {
            'status': response['FeedSubmissionInfo'][
                'FeedProcessingStatus'
            ]['value'],
            'submission_id': response['FeedSubmissionInfo'][
                'FeedSubmissionId'
            ]['value']
        }


class ExportAmazonPricingStart(ModelView):
    'Export Catalog Pricing to Amazon View'
    __name__ = 'amazon.export_catalog_pricing.start'


class ExportAmazonPricingDone(ModelView):
    'Export Catalog Pricing to Amazon Done View'
    __name__ = 'amazon.export_catalog_pricing.done'

    status = fields.Char('Status', readonly=True)
    submission_id = fields.Char('Submission ID', readonly=True)


class ExportAmazonPricing(Wizard):
    '''Export catalog pricing to Amazon

    Export the prices products selected to this amazon account
    '''
    __name__ = 'amazon.export_catalog_pricing'

    start = StateView(
        'amazon.export_catalog_pricing.start',
        'amazon_mws.export_catalog_pricing_start', [
            Button('Cancel', 'end', 'tryton-cancel'),
            Button('Continue', 'export_', 'tryton-ok', default=True),
        ]
    )
    export_ = StateTransition()
    done = StateView(
        'amazon.export_catalog_pricing.done',
        'amazon_mws.export_catalog_pricing_done', [
            Button('OK', 'end', 'tryton-cancel'),
        ]
    )

    def transition_export_(self):
        """
        Export the prices for products selected to this amazon account
        """
        SaleChannel = Pool().get('sale.channel')

        amazon_channel = SaleChannel(Transaction().context.get('active_id'))

        response = amazon_channel.export_pricing_to_amazon()

        Transaction().set_context({'response': response})

        return 'done'

    def default_done(self, fields):
        "Display response"
        response = Transaction().context['response']
        return {
            'status': response['FeedSubmissionInfo'][
                'FeedProcessingStatus'
            ]['value'],
            'submission_id': response['FeedSubmissionInfo'][
                'FeedSubmissionId'
            ]['value']
        }


class ExportAmazonInventoryStart(ModelView):
    'Export Catalog Inventory to Amazon View'
    __name__ = 'amazon.export_catalog_inventory.start'


class ExportAmazonInventoryDone(ModelView):
    'Export Catalog Inventory to Amazon Done View'
    __name__ = 'amazon.export_catalog_inventory.done'

    status = fields.Char('Status', readonly=True)
    submission_id = fields.Char('Submission ID', readonly=True)


class ExportAmazonInventory(Wizard):
    '''Export catalog inventory to Amazon

    Export the prices products selected to this amazon account
    '''
    __name__ = 'amazon.export_catalog_inventory'

    start = StateView(
        'amazon.export_catalog_inventory.start',
        'amazon_mws.export_catalog_inventory_start', [
            Button('Cancel', 'end', 'tryton-cancel'),
            Button('Continue', 'export_', 'tryton-ok', default=True),
        ]
    )
    export_ = StateTransition()
    done = StateView(
        'amazon.export_catalog_inventory.done',
        'amazon_mws.export_catalog_inventory_done', [
            Button('OK', 'end', 'tryton-cancel'),
        ]
    )

    def transition_export_(self):
        """
        Export the prices for products selected to this amazon account
        """
        SaleChannel = Pool().get('sale.channel')

        amazon_channel = SaleChannel(Transaction().context.get('active_id'))

        response = amazon_channel.export_inventory_to_amazon()

        Transaction().set_context({'response': response})

        return 'done'

    def default_done(self, fields):
        "Display response"
        response = Transaction().context['response']
        return {
            'status': response['FeedSubmissionInfo'][
                'FeedProcessingStatus'
            ]['value'],
            'submission_id': response['FeedSubmissionInfo'][
                'FeedSubmissionId'
            ]['value']
        }



# -*- coding: utf-8 -*-
"""
    channle.py

    :copyright: (c) 2015 by Openlabs Technologies & Consulting (P) Limited
    :license: BSD, see LICENSE for more details.
"""
from datetime import datetime
from dateutil.relativedelta import relativedelta
from mws import mws
from lxml import etree
from lxml.builder import E

from trytond.model import ModelView, fields
from trytond.wizard import Wizard, StateView, Button, StateAction
from trytond.transaction import Transaction
from trytond.pyson import Eval
from trytond.pool import Pool, PoolMeta
from trytond.pyson import PYSONEncoder

__metaclass__ = PoolMeta

__all__ = [
    'SaleChannel', 'CheckAmazonServiceStatusView', 'CheckAmazonServiceStatus',
    'CheckAmazonSettingsView', 'CheckAmazonSettings', 'ImportAmazonOrdersView',
    'ImportAmazonOrders',
]

AMAZON_MWS_STATES = {
    'required': Eval('source') == 'amazon_mws',
    'invisible': ~(Eval('source') == 'amazon_mws')
}


class SaleChannel:
    "Amazon MWS Account"
    __name__ = 'sale.channel'

    # These are the credentials that you receive when you register a seller
    # account with Amazon MWS
    amazon_merchant_id = fields.Char(
        "Merchant ID", states=AMAZON_MWS_STATES, depends=['source']
    )
    amazon_marketplace_id = fields.Char(
        "MarketPlace ID", states=AMAZON_MWS_STATES, depends=['source']
    )
    amazon_access_key = fields.Char(
        "Access Key", states=AMAZON_MWS_STATES, depends=['source']
    )
    amazon_secret_key = fields.Char(
        "Secret Key", states=AMAZON_MWS_STATES, depends=['source']
    )

    last_amazon_order_import_time = fields.DateTime(
        'Last Amazon Order Import Time', states={
            'invisible': ~(Eval('source') == 'amazon_mws')
        }, depends=['source']
    )

    last_amazon_product_export_time = fields.DateTime(
        'Last Amazon Product Export Time', states={
            'invisible': ~(Eval('source') == 'amazon_mws')
        }, depends=['source']
    )

    @staticmethod
    def default_last_amazon_order_import_time():
        """
        Sets default last order import time
        """
        return datetime.utcnow() - relativedelta(days=30)

    @classmethod
    def get_source(cls):
        """
        Get the source
        """
        sources = super(SaleChannel, cls).get_source()

        sources.append(('amazon_mws', 'Amazon Marketplace'))

        return sources

    @staticmethod
    def default_default_uom():
        UoM = Pool().get('product.uom')

        unit = UoM.search([
            ('name', '=', 'Unit'),
        ])
        return unit and unit[0].id or None

    @classmethod
    def __setup__(cls):
        """
        Setup the class before adding to pool
        """
        super(SaleChannel, cls).__setup__()
        cls._buttons.update({
            'check_amazon_service_status': {},
            'check_amazon_settings': {},
            'import_amazon_orders_button': {},
        })

        cls._error_messages.update({
            'orders_not_found': 'No orders seems to be placed after %s',
            "missing_product_codes": (
                'Product "%(product)s" misses Amazon Product Identifiers'
            ),
            "missing_product_code": (
                'Product "%(product)s" misses Product Code'
            ),
            'invalid_channel': 'Channel does not belong to Amazon.'
        })

    def validate_amazon_channel(self):
        """
        Check if channel belongs to amazon mws
        """
        if self.source != 'amazon_mws':
            self.raise_user_error('invalid_channel')

    def get_mws_api(self):
        """
        Create an instance of mws api

        :return: mws api instance
        """
        return mws.MWS(
            access_key=self.amazon_access_key,
            secret_key=self.amazon_secret_key,
            account_id=self.amazon_merchant_id,
        )

    def get_amazon_order_api(self):
        """
        Create an instance of Order api

        :return: order api instance
        """
        return mws.Orders(
            access_key=self.amazon_access_key,
            secret_key=self.amazon_secret_key,
            account_id=self.amazon_merchant_id,
        )

    def get_amazon_product_api(self):
        """
        Create an instance of product api

        :return: Product API instance
        """
        return mws.Products(
            access_key=self.amazon_access_key,
            secret_key=self.amazon_secret_key,
            account_id=self.amazon_merchant_id,
        )

    def get_amazon_feed_api(self):
        """
        Return an instance of feed api
        """
        return mws.Feeds(
            access_key=self.amazon_access_key,
            secret_key=self.amazon_secret_key,
            account_id=self.amazon_merchant_id,
        )

    @classmethod
    @ModelView.button_action('amazon_mws.check_amazon_service_status')
    def check_amazon_service_status(cls, channels):
        """
        Check GREEN, GREEN_I, YELLOW or RED status

        :param channels: Active record list of sale channels
        """
        pass

    @classmethod
    @ModelView.button_action('amazon_mws.check_amazon_settings')
    def check_amazon_settings(cls, channels):
        """
        Checks account settings configured

        :param accounts: Active record list of sale channels
        """
        pass

    @classmethod
    def import_amazon_orders_using_cron(cls):
        """
        Cron method to import amazon orders
        """
        channels = cls.search([('source', '=', 'amazon_mws')])

        for channel in channels:
            channel.import_amazon_orders()

    def import_amazon_orders(self):
        """
        Import Orders for current channel
        """
        Sale = Pool().get('sale.sale')

        self.validate_amazon_channel()

        order_api = self.get_amazon_order_api()

        sales = []
        last_import_time = self.last_amazon_order_import_time.isoformat()

        response = order_api.list_orders(
            marketplaceids=[self.amazon_marketplace_id],
            created_after=last_import_time,
            orderstatus=('Unshipped', 'PartiallyShipped', 'Shipped')
        ).parsed

        if not response.get('Orders'):
            self.raise_user_error('orders_not_found', last_import_time)

        # Orders are returned as dictionary for single order and as
        # list for multiple orders.
        # Convert to list if dictionary is returned
        if isinstance(response['Orders']['Order'], dict):
            orders = [response['Orders']['Order']]
        else:
            orders = response['Orders']['Order']

        with Transaction().set_context({'current_channel': self.id}):
            for order_data in orders:
                sales.append(
                    Sale.find_or_create_using_amazon_id(
                        order_data['AmazonOrderId']['value']
                    )
                )

        # Update last order import time for channel
        self.write([self], {'last_amazon_order_import_time': datetime.utcnow()})

        return sales

    @classmethod
    @ModelView.button_action('amazon_mws.import_amazon_orders')
    def import_amazon_orders_button(cls, channels):
        """
        Import orders for current account
        """
        pass

    def _get_amazon_envelop(self, message_type, xml_list):
        """
        Returns amazon envelop for xml given
        """
        NS = "http://www.w3.org/2001/XMLSchema-instance"
        location_attribute = '{%s}noNamespaceSchemaLocation' % NS

        envelope_xml = E.AmazonEnvelope(
            E.Header(
                E.DocumentVersion('1.01'),
                E.MerchantIdentifier(self.amazon_merchant_id)
            ),
            E.MessageType(message_type),
            E.PurgeAndReplace('false'),
            *(xml for xml in xml_list)
        )
        envelope_xml.set(location_attribute, 'amznenvelope.xsd')

        return envelope_xml

    @classmethod
    def export_to_amazon_using_cron(cls):
        """
        Cron method to export product catalog to amazon
        """
        channels = cls.search([('source', '=', 'amazon_mws')])

        for channel in channels:
            channel.export_catalog_to_amazon(silent=True)

    @classmethod
    def export_prices_to_amazon_using_cron(cls):
        """
        Cron method to export product prices to amazon
        """
        channels = cls.search([('source', '=', 'amazon_mws')])

        for channel in channels:
            channel.export_pricing_to_amazon(silent=True)

    @classmethod
    def export_inventory_to_amazon_using_cron(cls):
        """
        Cron method to export product inventory to amazon
        """
        channels = cls.search([('source', '=', 'amazon_mws')])

        for channel in channels:
            channel.export_inventory_to_amazon(silent=True)

    def export_catalog_to_amazon(self, silent=False):
        """
        Export the products to the Amazon account in context
        """
        Product = Pool().get('product.product')

        self.validate_amazon_channel()

        domain = [
            ('template.export_to_amazon', '=', True),
            ('code', '!=', None),
            ('codes', 'not in', []),
        ]

        if self.last_amazon_product_export_time:
            domain.append(
                ('write_date', '>=', self.last_amazon_product_export_time)
            )

        products = Product.search(domain)

        products_xml = []
        for product in products:
            if not product.code:
                if silent:
                    return
                self.raise_user_error(
                    'missing_product_code', {
                        'product': product.template.name
                    }
                )
            if not product.codes:
                if silent:
                    return
                self.raise_user_error(
                    'missing_product_codes', {
                        'product': product.template.name
                    }
                )
            # Get the product's code to be set as standard ID to amazon
            product_standard_id = (
                product.asin or product.ean or product.upc or product.isbn
                or product.gtin
            )
            products_xml.append(E.Message(
                E.MessageID(str(product.id)),
                E.OperationType('Update'),
                E.Product(
                    E.SKU(product.code),
                    E.StandardProductID(
                        E.Type(product_standard_id.code_type.upper()),
                        E.Value(product_standard_id.code),
                    ),
                    E.DescriptionData(
                        E.Title(product.template.name),
                        E.Description(product.description),
                    ),
                    # Amazon needs this information so as to place the product
                    # under a category.
                    # FIXME: Either we need to create all that inside our
                    # system or figure out a way to get all that via API
                    E.ProductData(
                        E.Miscellaneous(
                            E.ProductType('Misc_Other'),
                        ),
                    ),
                )
            ))

        envelope_xml = self._get_amazon_envelop('Product', products_xml)

        feeds_api = self.get_amazon_feed_api()

        response = feeds_api.submit_feed(
            etree.tostring(envelope_xml),
            feed_type='_POST_PRODUCT_DATA_',
            marketplaceids=[self.amazon_marketplace_id]
        )

        # Update last product export time for channel
        self.write([self], {
            'last_amazon_product_export_time': datetime.utcnow()
        })

        Product.write(products, {
            'channel_listings': [
                ('create', [{
                    'channel': self.id,
                }])
            ]
        })

        return response.parsed

    def export_pricing_to_amazon(self, silent=False):
        """Export prices of the products to the Amazon account in context

        :param products: List of active records of products
        """
        Product = Pool().get('product.product')

        self.validate_amazon_channel()

        products = Product.search([
            ('code', '!=', None),
            ('codes', 'not in', []),
            ('channel_listings.channel', '=', self.id),
        ])

        pricing_xml = []
        for product in products:
            if self in [
                ch.channel for ch in product.channel_listings
            ]:
                pricing_xml.append(E.Message(
                    E.MessageID(str(product.id)),
                    E.OperationType('Update'),
                    E.Price(
                        E.SKU(product.code),
                        E.StandardPrice(
                            # TODO: Use a pricelist
                            str(product.template.list_price),
                            currency=self.company.currency.code
                        ),
                    )
                ))

        envelope_xml = self._get_amazon_envelop('Price', pricing_xml)

        feeds_api = self.get_amazon_feed_api()

        response = feeds_api.submit_feed(
            etree.tostring(envelope_xml),
            feed_type='_POST_PRODUCT_PRICING_DATA_',
            marketplaceids=[self.amazon_marketplace_id]
        )

        return response.parsed

    def export_inventory_to_amazon(self, silent=False):
        """Export inventory of the products to the Amazon account in context

        :param products: List of active records of products
        """
        Product = Pool().get('product.product')

        self.validate_amazon_channel()

        products = Product.search([
            ('code', '!=', None),
            ('codes', 'not in', []),
            ('channel_listings.channel', '=', self.id),
        ])

        inventory_xml = []
        for product in products:
            with Transaction().set_context({'locations': [self.warehouse.id]}):
                quantity = product.quantity

            if not quantity:
                continue

            if self in [
                ch.channel for ch in product.channel_listings
            ]:
                inventory_xml.append(E.Message(
                    E.MessageID(str(product.id)),
                    E.OperationType('Update'),
                    E.Inventory(
                        E.SKU(product.code),
                        E.Quantity(str(round(quantity))),
                        E.FulfillmentLatency(
                            str(product.template.delivery_time)
                        ),
                    )
                ))

        envelope_xml = self._get_amazon_envelop('Inventory', inventory_xml)

        feeds_api = self.get_amazon_feed_api()

        response = feeds_api.submit_feed(
            etree.tostring(envelope_xml),
            feed_type='_POST_INVENTORY_AVAILABILITY_DATA_',
            marketplaceids=[self.amazon_marketplace_id]
        )

        return response.parsed


class CheckAmazonServiceStatusView(ModelView):
    "Check Service Status View"
    __name__ = 'channel.check_amazon_service_status.view'

    status = fields.Char('Status', readonly=True)
    message = fields.Text("Message", readonly=True)


class CheckAmazonServiceStatus(Wizard):
    """
    Check Service Status Wizard

    Check service status for the current MWS account
    """
    __name__ = 'channel.check_amazon_service_status'

    start = StateView(
        'channel.check_amazon_service_status.view',
        'amazon_mws.check_amazon_service_status_view_form',
        [
            Button('OK', 'end', 'tryton-ok'),
        ]
    )

    def default_start(self, data):
        """
        Check the service status of the MWS account

        :param data: Wizard data
        """
        SaleChannel = Pool().get('sale.channel')

        channel = SaleChannel(Transaction().context.get('active_id'))

        res = {}
        api = channel.get_mws_api()
        response = api.get_service_status().parsed

        status = response['Status']['value']

        if status == 'GREEN':
            status_message = 'The service is operating normally. '

        elif status == 'GREEN_I':
            status_message = 'The service is operating normally. '

        elif status == 'YELLOW':
            status_message = 'The service is experiencing higher than ' + \
                'normal error rates or is operating with degraded performance. '
        else:
            status_message = 'The service is unavailable or experiencing ' + \
                'extremely high error rates. '

        res['status'] = status
        if not response.get('Messages'):
            res['message'] = status_message
            return res

        if isinstance(response['Messages']['Message'], dict):
            messages = [response['Messages']['Message']]
        else:
            messages = response['Messages']['Message']

        for message in messages:
            status_message = status_message + message['Text']['value'] + ' '
            res['message'] = status_message

        return res


class CheckAmazonSettingsView(ModelView):
    "Check Amazon Settings View"
    __name__ = 'channel.check_amazon_settings.view'

    status = fields.Text('Status', readonly=True)


class CheckAmazonSettings(Wizard):
    """
    Wizard to Check Amazon MWS Settings

    Check amazon settings configured for the current MWS account
    """
    __name__ = 'channel.check_amazon_settings'

    start = StateView(
        'channel.check_amazon_settings.view',
        'amazon_mws.check_amazon_settings_view_form',
        [
            Button('OK', 'end', 'tryton-ok'),
        ]
    )

    def default_start(self, data):
        """
        Check the amazon settings for the current account

        :param data: Wizard data
        """
        SaleChannel = Pool().get('sale.channel')

        channel = SaleChannel(Transaction().context.get('active_id'))

        channel.validate_amazon_channel()

        res = {}
        api = channel.get_amazon_feed_api()

        try:
            api.get_feed_submission_count().parsed
            res['status'] = 'Account settings have been configured correctly'

        except mws.MWSError:
            res['status'] = "Something went wrong. Please check account " + \
                "settings again"
        return res


class ImportAmazonOrdersView(ModelView):
    "Import Orders View"
    __name__ = 'channel.import_amazon_orders.view'

    message = fields.Text("Message", readonly=True)


class ImportAmazonOrders(Wizard):
    """
    Import Amazon Orders Wizard

    Import orders for the current amazon channel
    """
    __name__ = 'channel.import_amazon_orders'

    start = StateView(
        'channel.import_amazon_orders.view',
        'amazon_mws.import_amazon_orders_view_form',
        [
            Button('Cancel', 'end', 'tryton-cancel'),
            Button('Continue', 'import_', 'tryton-ok', default=True),
        ]
    )

    import_ = StateAction('sale.act_sale_form')

    def default_start(self, data):
        """
        Sets default data for wizard
        """
        return {
            'message':
                'This wizard will import orders for this seller '
                'account. It imports orders updated only after Last Order '
                'Import Time.'
        }

    def do_import_(self, action):
        """
        Import orders and open records created
        """
        SaleChannel = Pool().get('sale.channel')

        channel = SaleChannel(Transaction().context.get('active_id'))
        channel.validate_amazon_channel()

        sales = channel.import_amazon_orders()

        action['pyson_domain'] = PYSONEncoder().encode([
            ('id', 'in', map(int, sales))
        ])
        return action, {}

    def transition_import_(self):
        return 'end'



# -*- coding: utf-8 -*-
"""
    __init__

    Initialize module

    :copyright: (c) 2013-2015 by Openlabs Technologies & Consulting (P) Limited
    :license: BSD, see LICENSE for more details.
"""
from trytond.pool import Pool
from channel import (
    SaleChannel, CheckAmazonServiceStatus, CheckAmazonServiceStatusView,
    CheckAmazonSettingsView, CheckAmazonSettings, ImportAmazonOrdersView,
    ImportAmazonOrders
)
from product import (
    Product, ExportAmazonCatalogStart, ExportAmazonCatalog,
    ExportAmazonCatalogDone, ExportAmazonPricingStart, ExportAmazonPricing,
    ExportAmazonPricingDone, ExportAmazonInventoryStart,
    ExportAmazonInventory, ExportAmazonInventoryDone, ProductCode, Template
)
from sale import Sale
from party import Party, Address
from country import Subdivision


def register():
    """
    Register classes with pool
    """
    Pool.register(
        SaleChannel,
        Product,
        ProductCode,
        Template,
        ExportAmazonCatalogStart,
        ExportAmazonCatalogDone,
        ExportAmazonPricingStart,
        ExportAmazonPricingDone,
        ExportAmazonInventoryStart,
        ExportAmazonInventoryDone,
        CheckAmazonServiceStatusView,
        CheckAmazonSettingsView,
        ImportAmazonOrdersView,
        Sale,
        Party,
        Address,
        Subdivision,
        module='amazon_mws', type_='model'
    )
    Pool.register(
        CheckAmazonServiceStatus,
        CheckAmazonSettings,
        ExportAmazonCatalog,
        ExportAmazonPricing,
        ExportAmazonInventory,
        ImportAmazonOrders,
        module='amazon_mws', type_='wizard'
    )



# -*- coding: utf-8 -*-
"""
    test_sale

    Tests Sale

    :copyright: (c) 2015 by Openlabs Technologies & Consulting (P) Limited
    :license: BSD, see LICENSE for more details.
"""
import os
import sys
import unittest
DIR = os.path.abspath(os.path.normpath(
    os.path.join(
        __file__,
        '..', '..', '..', '..', '..', 'trytond'
    )
))
if os.path.isdir(DIR):
    sys.path.insert(0, os.path.dirname(DIR))

import trytond.tests.test_tryton
from trytond.tests.test_tryton import POOL, USER, DB_NAME, CONTEXT
from test_base import TestBase, load_json
from trytond.transaction import Transaction


class TestSale(TestBase):
    """
    Tests import of sale order
    """

    def test_0010_create_sale_using_amazon_data(self):
        """
        Tests creation of sale order using amazon data
        """
        Sale = POOL.get('sale.sale')
        Product = POOL.get('product.product')
        Party = POOL.get('party.party')
        ContactMechanism = POOL.get('party.contact_mechanism')

        with Transaction().start(DB_NAME, USER, CONTEXT):
            self.setup_defaults()

            with Transaction().set_context({
                'current_channel': self.sale_channel.id,
            }):

                orders = Sale.search([])
                self.assertEqual(len(orders), 0)

                order_data = load_json(
                    'orders', 'order_list'
                )['Orders']['Order']
                line_data = load_json(
                    'orders', 'order_items'
                )['OrderItems']['OrderItem']
                self.assertFalse(
                    Party.search([
                        ('name', '=', order_data['BuyerEmail']['value'])
                    ])
                )

                self.assertFalse(
                    ContactMechanism.search([
                        ('party.name', '=', order_data['BuyerEmail']['value']),
                        ('type', 'in', ['phone', 'mobile']),
                        ('value', '=',
                            order_data['ShippingAddress']['Phone']['value']),
                    ])
                )

                # Create product using sku
                product_data = load_json('products', 'product-1')
                product_data.update({
                    'Id': {
                        'value': line_data['SellerSKU']['value']
                    }
                })
                Product.create_using_amazon_data(product_data)

                with Transaction().set_context(company=self.company.id):
                    order = Sale.create_using_amazon_data(order_data, line_data)

                self.assertEqual(order.state, 'confirmed')

                orders = Sale.search([])
                self.assertEqual(len(orders), 1)
                self.assertTrue(
                    Party.search([
                        ('name', '=', order_data['BuyerEmail']['value'])
                    ])
                )

                party, = Party.search([
                    ('name', '=', order_data['BuyerEmail']['value'])
                ])

                # Address is created for party
                self.assertEqual(len(party.addresses), 1)

                # Phone is added to party
                self.assertTrue(
                    ContactMechanism.search([
                        ('party', '=', party),
                        ('type', 'in', ['phone', 'mobile']),
                        ('value', '=',
                            order_data['ShippingAddress']['Phone']['value']),
                    ])
                )
                address, = party.addresses
                self.assertEqual(
                    address.name, order_data['ShippingAddress']['Name']['value']
                )

                # Item lines + shipping line should be equal to lines on tryton
                self.assertEqual(len(order.lines), 2)

    def test_0020_check_matched_address_using_amazon_data(self):
        """
        Tests address if same address already exists
        """
        Party = POOL.get('party.party')
        Address = POOL.get('party.address')

        with Transaction().start(DB_NAME, USER, CONTEXT):
            self.setup_defaults()

            with Transaction().set_context({
                'current_channel': self.sale_channel.id,
            }):

                order_data = load_json(
                    'orders', 'order_list'
                )['Orders']['Order']

                address_data = order_data['ShippingAddress']

                party = Party.create_using_amazon_data({
                    'name': order_data['BuyerEmail']['value'],
                    'email': order_data['BuyerName']['value'],
                })

                self.assertFalse(
                    Address.search([
                        ('name', '=', address_data['Name']['value'])
                    ])
                )

                # Add address for party
                Address.find_or_create_for_party_using_amazon_data(
                    party, order_data['ShippingAddress']
                )
                self.assertTrue(
                    Address.search([
                        ('name', '=', address_data['Name']['value'])
                    ])
                )
                self.assertEqual(
                    Address.search([
                        ('name', '=', address_data['Name']['value'])
                    ], count=True), 1
                )

                # Add same address for party
                Address.find_or_create_for_party_using_amazon_data(
                    party, order_data['ShippingAddress']
                )

                # Now new address is created
                self.assertEqual(
                    Address.search([
                        ('name', '=', address_data['Name']['value'])
                    ], count=True), 1
                )

    def test_0030_create_duplicate_party(self):
        """
        Tests duplicate party is created with same amazon email
        """
        Party = POOL.get('party.party')

        with Transaction().start(DB_NAME, USER, CONTEXT):
            self.setup_defaults()

            with Transaction().set_context({
                'current_channel': self.sale_channel.id,
            }):

                order_data = load_json(
                    'orders', 'order_list'
                )['Orders']['Order']

                self.assertFalse(
                    Party.search([
                        ('name', '=', order_data['BuyerEmail']['value'])
                    ])
                )

                party1 = Party.find_or_create_using_amazon_data({
                    'name': order_data['BuyerEmail']['value'],
                    'email': order_data['BuyerName']['value'],
                })

                self.assertTrue(
                    Party.search([
                        ('name', '=', order_data['BuyerEmail']['value'])
                    ])
                )
                self.assertEqual(
                    Party.search([
                        ('name', '=', order_data['BuyerEmail']['value'])
                    ], count=True), 1
                )

                # Create party with same email again and it wont create
                # new one
                party2 = Party.find_or_create_using_amazon_data({
                    'name': order_data['BuyerEmail']['value'],
                    'email': order_data['BuyerName']['value'],
                })

                self.assertEqual(party1, party2)

                self.assertEqual(
                    Party.search([
                        ('name', '=', order_data['BuyerEmail']['value'])
                    ], count=True), 1
                )


def suite():
    """
    Test Suite
    """
    test_suite = trytond.tests.test_tryton.suite()
    test_suite.addTests(
        unittest.TestLoader().loadTestsFromTestCase(TestSale)
    )
    return test_suite

if __name__ == '__main__':
    unittest.TextTestRunner(verbosity=2).run(suite())



# -*- coding: utf-8 -*-
"""
    sale

    Sale

    :copyright: (c) 2015 by Openlabs Technologies & Consulting (P) Limited
    :license: BSD, see LICENSE for more details.
"""
import dateutil.parser
from decimal import Decimal

from trytond.model import fields
from trytond.transaction import Transaction
from trytond.pool import PoolMeta, Pool
from trytond.pyson import Eval


__all__ = ['Sale']
__metaclass__ = PoolMeta


class Sale:
    "Sale"
    __name__ = 'sale.sale'

    amazon_order_id = fields.Char(
        'Amazon Order ID', readonly=True, select=True,
        help="This is global and unique ID given to an order across whole"
        "amazon"
        "Warning: Editing this might result in duplicate orders on next"
        " import",
        states={
            'invisible': ~(Eval('channel_type') == 'amazon_mws')
        }, depends=['channel_type']
    )

    @classmethod
    def __setup__(cls):
        """
        Setup the class before adding to pool
        """
        super(Sale, cls).__setup__()

        cls._sql_constraints += [
            (
                'amazon_id_unique',
                'UNIQUE(amazon_order_id)',
                'Sale with this amazon Order ID already exists'
            )
        ]

    @classmethod
    def find_or_create_using_amazon_id(cls, order_id):
        """
        This method tries to find the sale with the order ID
        first and if not found it will fetch the info from amazon and
        create a new sale with the data from amazon using
        create_using_amazon_data

        :param order_id: Order ID from amazon
        :type order_id: string
        :returns: Active record of sale order created/found
        """
        SaleChannel = Pool().get('sale.channel')

        sales = cls.search([
            ('amazon_order_id', '=', order_id),
        ])
        if sales:
            return sales[0]

        amazon_channel = SaleChannel(
            Transaction().context['current_channel']
        )
        assert amazon_channel.source == 'amazon_mws'

        order_api = amazon_channel.get_amazon_order_api()

        order_data = order_api.get_order([order_id]).parsed

        order_line_data = order_api.list_order_items(
            order_data['Orders']['Order']['AmazonOrderId']['value']
        ).parsed

        return cls.create_using_amazon_data(
            order_data['Orders']['Order'],
            order_line_data['OrderItems']['OrderItem']
        )

    @classmethod
    def create_using_amazon_data(cls, order_data, line_data):
        """
        Create a sale using amazon data

        :param order_data: Order data from amazon
        :return: Active record of record created
        """
        Party = Pool().get('party.party')
        Address = Pool().get('party.address')
        SaleChannel = Pool().get('sale.channel')

        amazon_channel = SaleChannel(
            Transaction().context['current_channel']
        )
        assert amazon_channel.source == 'amazon_mws'

        party_values = {
            'name': order_data['BuyerEmail']['value'],
            'email': order_data['BuyerName']['value'],
        }
        party = Party.find_or_create_using_amazon_data(party_values)

        party.add_phone_using_amazon_data(
            order_data['ShippingAddress']['Phone']['value']
        )
        party_invoice_address = party_shipping_address = \
            Address.find_or_create_for_party_using_amazon_data(
                party, order_data['ShippingAddress']
            )

        sale = cls.get_sale_using_amazon_data(order_data, line_data)

        sale.party = party.id
        sale.invoice_address = party_invoice_address.id
        sale.shipment_address = party_shipping_address.id
        sale.channel = amazon_channel.id
        sale.save()

        # TODO: Handle Discounts
        # TODO: Handle Taxes

        # Assert that the order totals are same
        # Cases handled according to OrderStatus
        # XXX: Handle case of PartiallyShipped
        if order_data['OrderStatus']['value'] == 'Unshipped':
            assert sale.total_amount == Decimal(
                order_data['OrderTotal']['Amount']['value']) * Decimal(
                order_data['NumberOfItemsUnshipped']['value']
            )
        elif order_data['OrderStatus']['value'] == 'Shipped':
            assert sale.total_amount == Decimal(
                order_data['OrderTotal']['Amount']['value']) * Decimal(
                order_data['NumberOfItemsShipped']['value']
            )

        # We import only completed orders, so we can confirm them all
        cls.quote([sale])
        cls.confirm([sale])

        # TODO: Process the order for invoice as the payment info is received

        return sale

    @classmethod
    def get_sale_using_amazon_data(cls, order_data, line_data):
        """
        Returns sale for amazon order
        """
        Sale = Pool().get('sale.sale')
        Currency = Pool().get('currency.currency')
        currency, = Currency.search([
            ('code', '=', order_data['OrderTotal']['CurrencyCode']['value'])
        ], limit=1)

        return Sale(
            reference=order_data['AmazonOrderId']['value'],
            sale_date=dateutil.parser.parse(
                order_data['PurchaseDate']['value']
            ).date(),
            currency=currency.id,
            amazon_order_id=order_data['AmazonOrderId']['value'],
            lines=cls.get_item_line_data_using_amazon_data(line_data)
        )

    @classmethod
    def get_item_line_data_using_amazon_data(cls, line_data):
        """
        Make data for an item line from the amazon data.

        :param order_items: Order items
        :return: List of data of order lines in required format
        """
        Uom = Pool().get('product.uom')
        Product = Pool().get('product.product')
        SaleLine = Pool().get('sale.line')

        unit, = Uom.search([('name', '=', 'Unit')])

        # Order lines are returned as dictionary for single record and as list
        # for mulitple reocrds.
        # So convert to list if its dictionary
        if isinstance(line_data, dict):
            # If its a single line order, then the array will be dict
            order_items = [line_data]
        else:
            # In case of multi line orders, the transaction array will be
            # a list of dictionaries
            order_items = line_data

        sale_lines = []
        for order_item in order_items:
            sale_lines.append(
                SaleLine(
                    description=order_item['Title']['value'],
                    unit_price=Decimal(
                        order_item['ItemPrice']['Amount']['value']
                    ),
                    unit=unit.id,
                    quantity=Decimal(order_item['QuantityOrdered']['value']),
                    product=Product.find_or_create_using_amazon_sku(
                        order_item['SellerSKU']['value'],
                    ).id
                )
            )

            if order_item['ShippingPrice']['Amount']['value']:
                sale_lines.append(
                    cls.get_shipping_line_data_using_amazon_data(order_item)
                )

        return sale_lines

    @classmethod
    def get_shipping_line_data_using_amazon_data(cls, order_item):
        """
        Create a shipping line for the given sale using amazon data

        :param order_item: Order Data from amazon
        """
        SaleLine = Pool().get('sale.line')
        Uom = Pool().get('product.uom')

        unit, = Uom.search([('name', '=', 'Unit')])

        return SaleLine(
            description='Amazon Shipping and Handling',
            unit_price=Decimal(
                order_item['ShippingPrice']['Amount']['value']
            ),
            unit=unit.id,
            quantity=Decimal(
                order_item['QuantityOrdered']['value']
            ),  # XXX: Not sure about this if shipping charges must
                # be applied to each quantity ordered
        )



# -*- coding: utf-8 -*-
"""
    party

    Party

    :copyright: (c) 2015 by Openlabs Technologies & Consulting (P) Limited
    :license: BSD, see LICENSE for more details.
"""
from trytond.model import fields
from trytond.pool import PoolMeta, Pool


__all__ = ['Party', 'Address']
__metaclass__ = PoolMeta


class Party:
    "Party"
    __name__ = 'party.party'

    amazon_user_email = fields.Char(
        'Amazon Email', readonly=True,
        help="This is global and unique email given to a user across whole "
        "amazon. "
        "Warning: Editing this might result in duplication of parties on next"
        " import"
    )

    @classmethod
    def __setup__(cls):
        """
        Setup the class before adding to pool
        """
        super(Party, cls).__setup__()
        cls._error_messages.update({
            'account_not_found': 'Amazon Account does not exist in context',
        })

        cls._sql_constraints += [
            (
                'amazon_id_unique',
                'UNIQUE(amazon_user_email)',
                'Party with this amazon Order ID already exists'
            )
        ]

    @classmethod
    def find_or_create_using_amazon_data(cls, amazon_data):
        """
        Creates record of customer values sent by amazon

        :param amazon_data: Dictionary of values for customer sent by amazon
        :return: Active record of record created
        """
        parties = cls.search([
            ('amazon_user_email', '=', amazon_data['email']),
        ])
        if parties:
            return parties[0]

        return cls.create_using_amazon_data(amazon_data)

    @classmethod
    def create_using_amazon_data(cls, amazon_data):
        """
        Creates record of customer values sent by amazon

        :param amazon_data: Dictionary of values for customer sent by amazon
        :return: Active record of record created
        """
        return cls.create([{
            'name': amazon_data['name'],
            'amazon_user_email': amazon_data['email'],
            'contact_mechanisms': [
                ('create', [{
                    'email': amazon_data['email']
                }])
            ]
        }])[0]

    def add_phone_using_amazon_data(self, amazon_phone):
        """
        Add contact mechanism for party
        """
        ContactMechanism = Pool().get('party.contact_mechanism')

        if not ContactMechanism.search([
            ('party', '=', self.id),
            ('type', 'in', ['phone', 'mobile']),
            ('value', '=', amazon_phone),
        ]):
            ContactMechanism.create([{
                'party': self.id,
                'type': 'phone',
                'value': amazon_phone,
            }])


class Address:
    "Address"
    __name__ = 'party.address'

    def is_match_found(self, amazon_address):
        """
        Match the current address with the address fetched from amazon.
        Match all the fields of the address, i.e., streets, city, subdivision
        and country. For any deviation in any field, returns False.

        :param amazon_address: Amazon address instance
        :return: True if address matches else False
        """
        return all([
            self.name == amazon_address.name,
            self.street == amazon_address.street,
            self.streetbis == amazon_address.streetbis,
            self.zip == amazon_address.zip,
            self.city == amazon_address.city,
            self.country == amazon_address.country,
            self.subdivision == amazon_address.subdivision,
        ])

    @classmethod
    def find_or_create_for_party_using_amazon_data(cls, party, address_data):
        """
        Look for the address in tryton corresponding to the address_record.
        If found, return the same else create a new one and return that.

        :param party: Party active record
        :param address_data: Dictionary of address data from amazon
        :return: Active record of address created/found
        """
        amazon_address = cls.get_address_from_amazon_data(party, address_data)

        for address in party.addresses:
            if address.is_match_found(amazon_address):
                return address

        else:
            # Create new address
            amazon_address.save()
            return amazon_address

    @classmethod
    def get_address_from_amazon_data(cls, party, address_data):
        """
        Return address instance for data fetched from amazon
        """
        Address = Pool().get('party.address')
        Country = Pool().get('country.country')
        Subdivision = Pool().get('country.subdivision')

        country, = Country.search([
            ('code', '=', address_data['CountryCode']['value'])
        ], limit=1)
        subdivision = Subdivision.search_using_amazon_state(
            address_data['StateOrRegion']['value'], country
        )

        return Address(
            party=party.id,
            name=address_data['Name']['value'],
            street=address_data['AddressLine1']['value'],
            streetbis=(
                address_data.get('AddressLine2') and
                address_data['AddressLine2'].get('value') or None
            ),
            zip=address_data['PostalCode']['value'],
            city=address_data['City']['value'],
            country=country.id,
            subdivision=subdivision.id,
        )



# -*- coding: utf-8 -*-
"""
    test_base

    :copyright: (c) 2013-2015 by Openlabs Technologies & Consulting (P) Limited
    :license: BSD, see LICENSE for more details.
"""
import os
import json
import unittest
from datetime import datetime
from dateutil.relativedelta import relativedelta
from decimal import Decimal

import trytond.tests.test_tryton
from trytond.tests.test_tryton import POOL, USER
from trytond.transaction import Transaction


ROOT_JSON_FOLDER = os.path.join(
    os.path.dirname(os.path.abspath(__file__)), 'json'
)


def load_json(resource, filename):
    """Reads the json file from the filesystem and returns the json loaded as
    python objects

    On filesystem, the files are kept in this format:
        json----
              |
            resource----
                       |
                       filename

    :param resource: The magento resource for which the file has to be
                     fetched. It is same as the folder name in which the files
                     are kept.
    :param filename: The name of the file to be fethced without `.json`
                     extension.
    :returns: Loaded json from the contents of the file read.
    """
    file_path = os.path.join(
        ROOT_JSON_FOLDER, resource, str(filename)
    ) + '.json'

    return json.loads(open(file_path).read())


class TestBase(unittest.TestCase):
    """
    Setup basic defaults
    """

    def setUp(self):
        """
        Set up data used in the tests.
        this method is called before each test function execution.
        """
        trytond.tests.test_tryton.install_module('amazon_mws')

    def setup_defaults(self):
        """
        Setup default data
        """
        self.Uom = POOL.get('product.uom')
        self.Currency = POOL.get('currency.currency')
        self.Company = POOL.get('company.company')
        self.Party = POOL.get('party.party')
        self.Country = POOL.get('country.country')
        self.Subdivision = POOL.get('country.subdivision')
        self.AccountTemplate = POOL.get('account.account.template')
        self.Account = POOL.get('account.account')
        self.CreateChartAccount = POOL.get(
            'account.create_chart', type="wizard"
        )
        self.Location = POOL.get('stock.location')
        self.User = POOL.get('res.user')
        self.PaymentTerm = POOL.get('account.invoice.payment_term')
        self.FiscalYear = POOL.get('account.fiscalyear')
        self.PriceList = POOL.get('product.price_list')
        self.Sequence = POOL.get('ir.sequence')
        self.SequenceStrict = POOL.get('ir.sequence.strict')
        self.AccountConfiguration = POOL.get('account.configuration')
        self.Property = POOL.get('ir.property')
        self.ModelField = POOL.get('ir.model.field')
        self.SaleChannel = POOL.get('sale.channel')

        with Transaction().set_context(company=None):
            self.party, = self.Party.create([{
                'name': 'ABC',
            }])
            self.usd, = self.Currency.create([{
                'name': 'US Dollar',
                'code': 'USD',
                'symbol': '$',
            }])
            self.company, = self.Company.create([{
                'party': self.party.id,
                'currency': self.usd.id,
            }])

        self.User.write([self.User(USER)], {
            'main_company': self.company.id,
            'company': self.company.id,
        })

        date = datetime.utcnow().date()

        with Transaction().set_context(
            self.User.get_preferences(context_only=True)
        ):

            invoice_sequence, = self.SequenceStrict.create([{
                'name': '%s' % date.year,
                'code': 'account.invoice',
                'company': self.company.id,
            }])
            fiscal_year, = self.FiscalYear.create([{
                'name': '%s' % date.year,
                'start_date': date + relativedelta(month=1, day=1),
                'end_date': date + relativedelta(month=12, day=31),
                'company': self.company.id,
                'post_move_sequence': self.Sequence.create([{
                    'name': '%s' % date.year,
                    'code': 'account.move',
                    'company': self.company.id,
                }])[0].id,
                'out_invoice_sequence': invoice_sequence.id,
                'in_invoice_sequence': invoice_sequence.id,
                'out_credit_note_sequence': invoice_sequence.id,
                'in_credit_note_sequence': invoice_sequence.id,
            }])
            self.FiscalYear.create_period([fiscal_year])

            account_template, = self.AccountTemplate.search(
                [('parent', '=', None)]
            )
            session_id, _, _ = self.CreateChartAccount.create()
            create_chart = self.CreateChartAccount(session_id)
            create_chart.account.account_template = account_template
            create_chart.account.company = self.company.id
            create_chart.transition_create_account()

            revenue, = self.Account.search([
                ('kind', '=', 'revenue'),
                ('company', '=', self.company.id),
            ])
            receivable, = self.Account.search([
                ('kind', '=', 'receivable'),
                ('company', '=', self.company.id),
            ])
            payable, = self.Account.search([
                ('kind', '=', 'payable'),
                ('company', '=', self.company.id),
            ])
            expense, = self.Account.search([
                ('kind', '=', 'expense'),
                ('company', '=', self.company.id),
            ])

            create_chart.properties.company = self.company
            create_chart.properties.account_receivable = receivable
            create_chart.properties.account_payable = payable
            create_chart.properties.account_revenue = revenue
            create_chart.properties.account_expense = expense
            create_chart.transition_create_properties()

        self.AccountConfiguration.write(
            [self.AccountConfiguration(1)], {
                'default_account_receivable': receivable.id,
                'default_account_payable': payable.id
            })

        self.Party.write(
            [self.party], {
                'account_payable': payable.id,
                'account_receivable': receivable.id,
            }
        )

        # Create payment term
        self.payment_term, = self.PaymentTerm.create([{
            'name': 'Direct',
            'lines': [('create', [{'type': 'remainder'}])]
        }])

        # Search product uom
        self.uom, = self.Uom.search([
            ('name', '=', 'Unit'),
        ])

        self.country1, = self.Country.create([{
            'name': 'United States',
            'code': 'US',
        }])

        self.country2, = self.Country.create([{
            'name': 'India',
            'code': 'IN',
        }])

        self.subdivision1, = self.Subdivision.create([{
            'name': 'Florida',
            'code': 'US-FL',
            'country': self.country1.id,
            'type': 'state',
        }, ])

        self.subdivision2, = self.Subdivision.create([{
            'name': 'Uttar Pradesh',
            'code': 'IN-UP',
            'country': self.country2.id,
            'type': 'state',
        }])

        self.subdivision3, self.subdivision4 = self.Subdivision.create([
            {
                'name': 'American Samoa',
                'code': 'US-AS',
                'type': 'state',
                'country': self.country1.id,
            }, {
                'name': 'Alabama',
                'code': 'US-AL',
                'type': 'state',
                'country': self.country1.id,
            }
        ])

        self.price_list = self._create_pricelists()[1]

        warehouse, = self.Location.search([
            ('type', '=', 'warehouse')
        ], limit=1)

        self.sale_channel, = self.SaleChannel.create([{
            'name': 'Amazon MWS Account',
            'amazon_merchant_id': '1234',
            'amazon_marketplace_id': '3456',
            'amazon_access_key': 'AWS1',
            'amazon_secret_key': 'S013',
            'warehouse': warehouse.id,
            'company': self.company.id,
            'source': 'amazon_mws',
            'currency': self.company.currency.id,
            'default_account_revenue': self.get_account_by_kind('revenue'),
            'default_account_expense': self.get_account_by_kind('expense'),
            'default_uom': self.uom,
            'price_list': self.price_list,
            'invoice_method': 'manual',
            'shipment_method': 'manual',
            'payment_term': self.payment_term,
        }])

        model_field, = self.ModelField.search([
            ('name', '=', 'account_revenue'),
            ('model.model', '=', 'product.template'),
        ], order=[], limit=1)

        # TODO: This should work without creating new properties
        self.Property.create([{
            'value': 'account.account' + ',' + str(
                self.sale_channel.default_account_revenue.id
            ),
            'res': None,
            'field': model_field.id,
        }])

    def get_account_by_kind(self, kind, company=None, silent=True):
        """Returns an account with given spec

        :param kind: receivable/payable/expense/revenue
        :param silent: dont raise error if account is not found
        """
        Account = POOL.get('account.account')
        Company = POOL.get('company.company')

        if company is None:
            company, = Company.search([], limit=1)

        accounts = Account.search([
            ('kind', '=', kind),
            ('company', '=', company.id)
        ], limit=1)
        if not accounts and not silent:
            raise Exception("Account not found")
        return accounts and accounts[0].id or None

    def _create_pricelists(self):
        """
        Create the pricelists
        """
        # Setup the pricelists
        self.party_pl_margin = Decimal('1.10')
        self.guest_pl_margin = Decimal('1.20')
        user_price_list, = self.PriceList.create([{
            'name': 'PL 1',
            'company': self.company.id,
            'lines': [
                ('create', [{
                    'formula': 'unit_price * %s' % self.party_pl_margin
                }])
            ],
        }])
        guest_price_list, = self.PriceList.create([{
            'name': 'PL 2',
            'company': self.company.id,
            'lines': [
                ('create', [{
                    'formula': 'unit_price * %s' % self.guest_pl_margin
                }])
            ],
        }])
        return guest_price_list.id, user_price_list.id



# -*- coding: utf-8 -*-
"""
    test_product

    Tests Product

    :copyright: (c) 2013 by Openlabs Technologies & Consulting (P) Limited
    :license: BSD, see LICENSE for more details.
"""
import sys
import os
from decimal import Decimal
DIR = os.path.abspath(os.path.normpath(
    os.path.join(
        __file__,
        '..', '..', '..', '..', '..', 'trytond'
    )
))
if os.path.isdir(DIR):
    sys.path.insert(0, os.path.dirname(DIR))

import unittest
import trytond.tests.test_tryton
from trytond.tests.test_tryton import POOL, USER, DB_NAME, CONTEXT
from trytond.transaction import Transaction
from test_base import TestBase, load_json


class TestProduct(TestBase):
    '''
    Tests Product
    '''

    def test_0010_code_fields(self):
        """Tests the function fields for codes
        """
        Template = POOL.get('product.template')

        with Transaction().start(DB_NAME, USER, CONTEXT):
            self.setup_defaults()

            template, = Template.create([{
                'name': 'Test Product',
                'default_uom': self.uom.id,
                'list_price': Decimal('10.0'),
                'cost_price': Decimal('8.0'),
                'account_expense': self.get_account_by_kind('expense'),
                'account_revenue': self.get_account_by_kind('revenue'),
                'export_to_amazon': True,
                'products': [('create', [{
                    'code': 'code1',
                    'description': 'Some product description',
                    'codes': [('create', [{
                        'code': 'BUYGBS6866',
                        'code_type': 'asin',
                    }, {
                        'code': '123456789012',
                        'code_type': 'upc',
                    }, {
                        'code': '1234567890123',
                        'code_type': 'ean',
                    }])]
                }])]
            }])

            product, = template.products

            self.assertEqual(product.ean.code, '1234567890123')
            self.assertEqual(product.upc.code, '123456789012')
            self.assertEqual(product.asin.code, 'BUYGBS6866')

    def test_0020_create_product_using_amazon_data(self):
        """
        Tests if product is created using amazon data
        """
        Product = POOL.get('product.product')

        with Transaction().start(DB_NAME, USER, CONTEXT):
            self.setup_defaults()

            with Transaction().set_context(
                {'current_channel': self.sale_channel.id}
            ):
                self.assertEqual(Product.search([], count=True), 0)

                product_data = load_json('products', 'product-1')
                Product.create_using_amazon_data(product_data)

                self.assertEqual(Product.search([], count=True), 1)


def suite():
    """
    Test Suite
    """
    test_suite = trytond.tests.test_tryton.suite()
    test_suite.addTests(
        unittest.TestLoader().loadTestsFromTestCase(TestProduct)
    )
    return test_suite

if __name__ == '__main__':
    unittest.TextTestRunner(verbosity=2).run(suite())



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
    setup

    Setup Module

    :copyright: (c) 2013-2015 by Openlabs Technologies & Consulting (P) Limited
    :license: BSD, see LICENSE for more details.
"""
from setuptools import setup, Command
import re
import os
import ConfigParser
import sys
import unittest
import time


class SQLiteTest(Command):
    """
    Run the tests on SQLite
    """
    description = "Run tests on SQLite"

    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        os.environ['TRYTOND_DATABASE_URI'] = 'sqlite://'
        os.environ['DB_NAME'] = ':memory:'

        from tests import suite
        test_result = unittest.TextTestRunner(verbosity=3).run(suite())

        if test_result.wasSuccessful():
            sys.exit(0)
        sys.exit(-1)


class PostgresTest(Command):
    """
    Run the tests on Postgres.
    """
    description = "Run tests on Postgresql"

    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        if self.distribution.tests_require:
            self.distribution.fetch_build_eggs(self.distribution.tests_require)

        os.environ['TRYTOND_DATABASE_URI'] = 'postgresql://'

        os.environ['DB_NAME'] = 'test_' + str(int(time.time()))

        from tests import suite
        test_result = unittest.TextTestRunner(verbosity=3).run(suite())

        if test_result.wasSuccessful():
            sys.exit(0)
        sys.exit(-1)


def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

config = ConfigParser.ConfigParser()
config.readfp(open('tryton.cfg'))
info = dict(config.items('tryton'))
for key in ('depends', 'extras_depend', 'xml'):
    if key in info:
        info[key] = info[key].strip().splitlines()
major_version, minor_version, _ = info.get('version', '0.0.1').split('.', 2)
major_version = int(major_version)
minor_version = int(minor_version)

requires = ['python-amazon-mws']

MODULE2PREFIX = {
    'sale_channel': 'openlabs'
}
MODULE = "amazon_mws"
PREFIX = "openlabs"
for dep in info.get('depends', []):
    if not re.match(r'(ir|res|webdav)(\W|$)', dep):
        requires.append(
            '%s_%s >= %s.%s, < %s.%s' % (
                MODULE2PREFIX.get(dep, 'trytond'), dep,
                major_version, minor_version, major_version,
                minor_version + 1
            )
        )

setup(
    name='%s_%s' % (PREFIX, MODULE),
    version=info.get('version', '0.0.1'),
    description='Amazon MWS Integration',
    long_description=read('README.rst'),
    author='Openlabs Technologies and Consulting P Ltd.',
    url='http://openlabs.co.in/',
    download_url="https://github.com/openlabs/trytond-amazon-mws",
    package_dir={'trytond.modules.%s' % MODULE: '.'},
    packages=[
        'trytond.modules.%s' % MODULE,
        'trytond.modules.%s.tests' % MODULE,
    ],
    package_data={
        'trytond.modules.%s' % MODULE:
            info.get('xml', []) + [
                'tryton.cfg', 'view/*xml'
            ],
    },
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Plugins',
        'Framework :: Tryton',
        'Intended Audience :: Developers',
        'Intended Audience :: Financial and Insurance Industry',
        'Intended Audience :: Legal Industry',
        'Intended Audience :: Manufacturing',
        'License :: OSI Approved :: BSD License',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2.7',
        'Topic :: Office/Business',
    ],
    license='GPL-3',
    install_requires=requires,
    tests_require=[],
    zip_safe=False,
    entry_points="""
    [trytond.modules]
    %s = trytond.modules.%s
    """ % (MODULE, MODULE),
    test_suite='tests',
    test_loader='trytond.test_loader:Loader',
    cmdclass={
        'test': SQLiteTest,
        'test_on_postgres': PostgresTest,
    },
)
